@@ -62,8 +62,9 @@ struct Parse_Options_s
 {
 	int verbosity;         /* Level of detail to give about the computation 0 */
 	int use_sat_solver;    /* Use the Boolean SAT based parser */
-	int linkage_limit;     /* The maximum number of linkages processed 10000 */
+	int linkage_limit;     /* The maximum number of linkages processed 100 */
 	float disjunct_cost;   /* Max disjunct cost to allow */
+	int use_fat_links;     /* Look for fat linkages */
 	int min_null_count;    /* The minimum number of null links to allow */
 	int max_null_count;    /* The maximum number of null links to allow */
 	int null_block;        /* consecutive blocks of this many words are
@@ -208,16 +209,17 @@ struct Parse_info_struct
 	X_table_connector ** x_table;
 	Parse_set *    parse_set;
 	int            N_words;
-	Disjunct *     chosen_disjuncts[MAX_SENTENCE];
+	Disjunct **    chosen_disjuncts;
 	int            N_links;
 	Link          link_array[MAX_LINKS];
 
 	/* Points to the image structure for each word.
 	 * NULL if not a fat word. */
-	Image_node * image_array[MAX_SENTENCE];
+	Image_node ** image_array;
 
-	/* TRUE if this word has a fat down link. FALSE otherise */
-	char has_fat_down[MAX_SENTENCE];
+	/* Array of boolean flags, one per word. Set to TRUE if this 
+	 * word has a fat down link. FALSE otherise */
+	Boolean *has_fat_down;
 
 	/* thread-safe random number state */
 	unsigned int rand_state;
@@ -229,21 +231,23 @@ struct Sentence_s
 	const char *orig_sentence;  /* Copy of original sentence */
 	int    length;              /* number of words */
 	Word   word[MAX_SENTENCE];  /* array of words after tokenization */
-	char * is_conjunction;      /* TRUE if conjunction, as defined by dictionary */
+	char * is_conjunction;      /* Array of flags, one per word; set to
+	                               TRUE if conjunction, as defined by dictionary */
 	char** deletable;           /* deletable regions in a sentence with conjunction */
 	char** dptr;                /* private pointer for mem management only */
 	char** effective_dist;
 	int    num_linkages_found;  /* total number before postprocessing.  This
-				   is returned by the count() function */
+	                               is returned by the count() function */
 	int    num_linkages_alloced;/* total number of linkages allocated.
-				   the number post-processed might be fewer
-				   because some are non-canonical */
+	                               the number post-processed might be fewer
+	                               because some are non-canonical */
 	int    num_linkages_post_processed;
 	                            /* The number of linkages that are actually
-				   put into the array that was alloced.
-				   this is not the same as num alloced
-				   because some may be non-canonical. */
+	                               put into the array that was alloced.
+	                               This is not the same as num alloced
+	                               because some may be non-canonical. */
 	int    num_valid_linkages;  /* number with no pp violations */
+	int    num_thin_linkages;   /* valid linkages which are not fat */
 	int    null_links;          /* null links allowed */
 	int    null_count;          /* number of null links in linkages */
 	Parse_info     parse_info;  /* set of parses for the sentence */
