@@ -39,6 +39,9 @@ static int VDAL_compare_parse(Linkage_info * p1, Linkage_info * p2)
 	else if (p1->unused_word_cost != p2->unused_word_cost) {
 		return (p1->unused_word_cost - p2->unused_word_cost);
 	}
+	else if (p1->fat != p2->fat) {
+		return (p1->fat - p2->fat);
+	}
 	else if (p1->disjunct_cost != p2->disjunct_cost) {
 		return (p1->disjunct_cost - p2->disjunct_cost);
 	}
@@ -75,6 +78,7 @@ Parse_Options parse_options_create(void)
 	po->verbosity	 = 1;
 	po->linkage_limit = 100;
 	po->disjunct_cost = MAX_DISJUNCT_COST;
+	po->use_fat_links = FALSE;
 	po->min_null_count = 0;
 	po->max_null_count = 0;
 	po->null_block = 1;
@@ -175,6 +179,13 @@ int parse_options_get_use_sat_parser(Parse_Options opts) {
 	return opts->use_sat_solver;
 }
 
+void parse_options_set_use_fat_links(Parse_Options opts, int dummy) {
+	opts->use_fat_links = dummy;
+}
+int parse_options_get_use_fat_links(Parse_Options opts) {
+	return opts->use_fat_links;
+}
+
 void parse_options_set_linkage_limit(Parse_Options opts, int dummy) {
 	opts->linkage_limit = dummy;
 }
@@ -838,6 +849,7 @@ static void post_process_linkages(Sentence sent, Parse_Options opts)
 	int in, block_bottom, block_top;
 	int N_linkages_found, N_linkages_alloced;
 	int N_linkages_post_processed, N_valid_linkages;
+	int N_thin_linkages;
 	int overflowed, only_canonical_allowed;
 	Linkage_info *link_info;
 	int canonical;
@@ -867,6 +879,7 @@ static void post_process_linkages(Sentence sent, Parse_Options opts)
 		sent->num_linkages_alloced = 0;
 		sent->num_linkages_post_processed = 0;
 		sent->num_valid_linkages = 0;
+		sent->num_thin_linkages = 0;
 		sent->link_info = NULL;
 		return;
 	}
@@ -947,6 +960,7 @@ static void post_process_linkages(Sentence sent, Parse_Options opts)
 
 	/* second pass: actually perform post-processing */
 	N_linkages_post_processed = 0;
+	N_thin_linkages = 0;
 	for (in=0; (in < N_linkages_alloced) &&
 			   (!resources_exhausted(opts->resources)); in++)
 	{
@@ -969,6 +983,7 @@ static void post_process_linkages(Sentence sent, Parse_Options opts)
 		if (0 == lifo->N_violations)
 		{
 			N_valid_linkages++;
+			if (FALSE == lifo->fat) N_thin_linkages++;
 		}
 		lifo->index = indices[in];
 		lg_corpus_score(sent, lifo);
@@ -1016,6 +1031,7 @@ static void post_process_linkages(Sentence sent, Parse_Options opts)
 	sent->num_linkages_alloced = N_linkages_alloced;
 	sent->num_linkages_post_processed = N_linkages_post_processed;
 	sent->num_valid_linkages = N_valid_linkages;
+	sent->num_thin_linkages = N_thin_linkages;
 	sent->link_info = link_info;
 
 	xfree(indices, N_linkages_alloced * sizeof(int));
@@ -1132,7 +1148,7 @@ void sentence_delete(Sentence sent)
 	free_sentence_disjuncts(sent);
 	free_sentence_expressions(sent);
 	string_set_delete(sent->string_set);
-	free_parse_set(sent);
+	if (sent->parse_info) free_parse_info(sent->parse_info);
 	free_post_processing(sent);
 	post_process_close_sentence(sent->dict->postprocessor);
 	free_deletable(sent);
@@ -1166,6 +1182,11 @@ int sentence_null_count(Sentence sent) {
 	return sent->null_count;
 }
 
+int sentence_num_thin_linkages(Sentence sent) {
+	if (!sent) return 0;
+	return sent->num_thin_linkages;
+}
+
 int sentence_num_linkages_found(Sentence sent) {
 	if (!sent) return 0;
 	return sent->num_linkages_found;
@@ -1220,8 +1241,8 @@ static void chart_parse(Sentence sent, Parse_Options opts)
 	/* A parse set may have been already been built for this sentence,
 	 * if it was previously parsed.  If so we free it up before
 	 * building another.  */
-	free_parse_set(sent);
-	init_x_table(sent);
+	if (sent->parse_info) free_parse_info(sent->parse_info);
+	sent->parse_info = parse_info_new(sent->length);
 
 	for (nl = opts->min_null_count; nl<=opts->max_null_count ; ++nl)
 	{
@@ -1668,6 +1689,11 @@ int linkage_disjunct_cost(Linkage linkage)
 	return (int) floorf(linkage->info->disjunct_cost);
 }
 
+int linkage_is_fat(Linkage linkage)
+{
+	return linkage->info->fat;
+}
+
 int linkage_and_cost(Linkage linkage)
 {
 	return linkage->info->and_cost;
