@@ -33,7 +33,7 @@ typedef struct
 } per_thread_data;
 
 /* XXX FIXME
- * The per_thread_data struct should ideally be somehow 
+ * The per_thread_data struct should ideally be somehow
  * fetched from JNIEnv, or as an opaque pointer in the class.
  * Not clear how to do this .. perhaps use NewDirectByteBuffer()
  * and the java.nio.ByteBuffer class ?
@@ -72,11 +72,12 @@ static void setup_panic_parse_options(Parse_Options opts)
 	parse_options_set_min_null_count(opts, 1);
 	parse_options_set_max_null_count(opts, MAX_SENTENCE);
 	parse_options_set_max_parse_time(opts, 60);
-	parse_options_set_islands_ok(opts, 1);
+	parse_options_set_use_fat_links(opts, FALSE);
+	parse_options_set_islands_ok(opts, TRUE);
 	parse_options_set_short_length(opts, 6);
-	parse_options_set_all_short_connectors(opts, 1);
+	parse_options_set_all_short_connectors(opts, TRUE);
 	parse_options_set_linkage_limit(opts, 100);
-	parse_options_set_verbosity(opts,0);
+	parse_options_set_verbosity(opts, 0);
 	parse_options_set_spell_guess(opts, FALSE);
 }
 
@@ -97,7 +98,6 @@ static void throwException(JNIEnv *env, const char* message)
 	jclass exceptionClazz;
 	if ((*env)->ExceptionOccurred(env) != NULL) return;
 
-	
 	msg = (char *) malloc(50+strlen(message));
 	strcpy(msg, "link-parser:\n");
 	strcat(msg, message);
@@ -130,7 +130,7 @@ static per_thread_data * init(JNIEnv *env, jclass cls)
 	parse_options_set_verbosity(ptd->opts,0);
 	parse_options_set_spell_guess(ptd->opts, FALSE);
 
-	/* Default to the english language; will need to fix 
+	/* Default to the english language; will need to fix
 	 * this if/when more languages are supported.
 	 */
 	ptd->dict = dictionary_create_lang("en");
@@ -239,15 +239,23 @@ static void jParse(JNIEnv *env, per_thread_data *ptd, char* inputString)
 		return;
 	}
 
-	/* First parse with cost 0 or 1 and no null links */
+	/* First parse with cost 0 or 1 and no null links or fat links */
 	parse_options_set_disjunct_costf(opts, 2.0f);
 	parse_options_set_min_null_count(opts, 0);
 	parse_options_set_max_null_count(opts, 0);
+	parse_options_set_use_fat_links(opts, FALSE);
 	parse_options_reset_resources(opts);
 
 	ptd->num_linkages = sentence_parse(ptd->sent, ptd->opts);
 
-	/* Now parse with null links */
+	/* If failed, try with fat links ... */
+	if ((0 == ptd->num_linkages) && sentence_contains_conjunction(ptd->sent))
+	{
+		parse_options_set_use_fat_links(opts, TRUE);
+		ptd->num_linkages = sentence_parse(ptd->sent, ptd->opts);
+	}
+
+	/* If still failed, try again with null links */
 	if (0 == ptd->num_linkages)
 	{
 		if (jverbosity > 0) prt_error("Warning: No complete linkages found.\n");
@@ -307,8 +315,8 @@ static void makeLinkage(per_thread_data *ptd)
 /* Java JNI wrappers */
 
 /*
- * Class:		 LinkGrammar
- * Method:		getVersion
+ * Class:      LinkGrammar
+ * Method:     getVersion
  * Signature: (I)Ljava/lang/String;
  */
 JNIEXPORT jstring JNICALL
@@ -322,7 +330,7 @@ Java_org_linkgrammar_LinkGrammar_getVersion(JNIEnv *env, jclass cls)
 JNIEXPORT jstring JNICALL
 Java_org_linkgrammar_LinkGrammar_getDictVersion(JNIEnv *env, jclass cls)
 {
-	per_thread_data *ptd = get_ptd(env, cls);;
+	per_thread_data *ptd = get_ptd(env, cls);
 	const char *s = linkgrammar_get_dict_version(ptd->dict);
 	jstring j = (*env)->NewStringUTF(env, s);
 	return j;
@@ -342,14 +350,14 @@ Java_org_linkgrammar_LinkGrammar_setMaxCost(JNIEnv *env, jclass cls, jint maxCos
 	parse_options_set_disjunct_cost(ptd->opts, maxCost);
 }
 
-JNIEXPORT void JNICALL 
+JNIEXPORT void JNICALL
 Java_org_linkgrammar_LinkGrammar_setDictionariesPath(JNIEnv *env,
                                               jclass cls, jstring path)
 {
 	const char *nativePath = (*env)->GetStringUTFChars(env,path, 0);
 
 	// Java passes null pointers as the string "null"
-	if (nativePath && strcmp(nativePath, "null")) 
+	if (nativePath && strcmp(nativePath, "null"))
 	{
 		dictionary_set_data_dir(nativePath);
 	}
@@ -357,8 +365,8 @@ Java_org_linkgrammar_LinkGrammar_setDictionariesPath(JNIEnv *env,
 }
 
 /*
- * Class:		 LinkGrammar
- * Method:		init
+ * Class:      LinkGrammar
+ * Method:     init
  * Signature: ()V
  */
 JNIEXPORT void JNICALL
@@ -378,8 +386,8 @@ Java_org_linkgrammar_LinkGrammar_init(JNIEnv *env, jclass cls)
 }
 
 /*
- * Class:		 LinkGrammar
- * Method:		parse
+ * Class:      LinkGrammar
+ * Method:     parse
  * Signature: (Ljava/lang/String;)V
  */
 JNIEXPORT void JNICALL
@@ -396,8 +404,8 @@ Java_org_linkgrammar_LinkGrammar_parse(JNIEnv *env, jclass cls, jstring str)
 }
 
 /*
- * Class:		 LinkGrammar
- * Method:		close
+ * Class:      LinkGrammar
+ * Method:     close
  * Signature: ()V
  */
 JNIEXPORT void JNICALL
@@ -408,8 +416,8 @@ Java_org_linkgrammar_LinkGrammar_close(JNIEnv *env, jclass cls)
 }
 
 /*
- * Class:		 LinkGrammar
- * Method:		numWords
+ * Class:      LinkGrammar
+ * Method:     numWords
  * Signature: ()I
  */
 JNIEXPORT jint JNICALL
@@ -420,8 +428,8 @@ Java_org_linkgrammar_LinkGrammar_getNumWords(JNIEnv *env, jclass cls)
 }
 
 /*
- * Class:		 LinkGrammar
- * Method:		getWord
+ * Class:      LinkGrammar
+ * Method:     getWord
  * Signature: (I)Ljava/lang/String;
  */
 JNIEXPORT jstring JNICALL
@@ -467,7 +475,7 @@ Java_org_linkgrammar_LinkGrammar_getLinkageDisjunct(JNIEnv *env, jclass cls, jin
 }
 
 JNIEXPORT jstring JNICALL
-Java_org_linkgrammar_LinkGrammar_getLinkageSense(JNIEnv *env, 
+Java_org_linkgrammar_LinkGrammar_getLinkageSense(JNIEnv *env,
                     jclass cls, jint i, jint j)
 {
 	per_thread_data *ptd = get_ptd(env, cls);
@@ -484,7 +492,7 @@ Java_org_linkgrammar_LinkGrammar_getLinkageSense(JNIEnv *env,
 		sns = lg_sense_next(sns);
 		j--;
 	}
-	
+
 	/* does not need to be freed, points into data structures */
 	if (sns) w = lg_sense_get_sense(sns);
 
@@ -494,7 +502,7 @@ Java_org_linkgrammar_LinkGrammar_getLinkageSense(JNIEnv *env,
 }
 
 JNIEXPORT jdouble JNICALL
-Java_org_linkgrammar_LinkGrammar_getLinkageSenseScore(JNIEnv *env, 
+Java_org_linkgrammar_LinkGrammar_getLinkageSenseScore(JNIEnv *env,
                     jclass cls, jint i, jint j)
 {
 	per_thread_data *ptd = get_ptd(env, cls);
@@ -509,15 +517,15 @@ Java_org_linkgrammar_LinkGrammar_getLinkageSenseScore(JNIEnv *env,
 		sns = lg_sense_next(sns);
 		j--;
 	}
-	
+
 	if (sns) score = lg_sense_get_score(sns);
 
 	return score;
 }
 
 /*
- * Class:		 LinkGrammar
- * Method:		numSkippedWords
+ * Class:      LinkGrammar
+ * Method:     numSkippedWords
  * Signature: ()I
  */
 JNIEXPORT jint JNICALL
@@ -528,8 +536,8 @@ Java_org_linkgrammar_LinkGrammar_getNumSkippedWords(JNIEnv *env, jclass cls)
 }
 
 /*
- * Class:		 LinkGrammar
- * Method:		numLinkages
+ * Class:      LinkGrammar
+ * Method:     numLinkages
  * Signature: ()I
  */
 JNIEXPORT jint JNICALL
@@ -540,8 +548,8 @@ Java_org_linkgrammar_LinkGrammar_getNumLinkages(JNIEnv *env, jclass cls)
 }
 
 /*
- * Class:		 LinkGrammar
- * Method:		makeLinkage
+ * Class:      LinkGrammar
+ * Method:     makeLinkage
  * Signature: (I)I
  */
 JNIEXPORT void JNICALL
@@ -553,8 +561,8 @@ Java_org_linkgrammar_LinkGrammar_makeLinkage(JNIEnv *env, jclass cls, jint i)
 }
 
 /*
- * Class:		 LinkGrammar
- * Method:		linkageNumViolations
+ * Class:      LinkGrammar
+ * Method:     linkageNumViolations
  * Signature: ()I
  */
 JNIEXPORT jint JNICALL
@@ -565,8 +573,8 @@ Java_org_linkgrammar_LinkGrammar_getLinkageNumViolations(JNIEnv *env, jclass cls
 }
 
 /*
- * Class:		 LinkGrammar
- * Method:		linkageAndCost
+ * Class:      LinkGrammar
+ * Method:     linkageAndCost
  * Signature: ()I
  */
 JNIEXPORT jint JNICALL
@@ -577,8 +585,8 @@ Java_org_linkgrammar_LinkGrammar_getLinkageAndCost(JNIEnv *env, jclass cls)
 }
 
 /*
- * Class:		 LinkGrammar
- * Method:		linkageDisjunctCost
+ * Class:      LinkGrammar
+ * Method:     linkageDisjunctCost
  * Signature: ()I
  */
 JNIEXPORT jint JNICALL
@@ -589,8 +597,8 @@ Java_org_linkgrammar_LinkGrammar_getLinkageDisjunctCost(JNIEnv *env, jclass cls)
 }
 
 /*
- * Class:		 LinkGrammar
- * Method:		linkageLinkCost
+ * Class:      LinkGrammar
+ * Method:     linkageLinkCost
  * Signature: ()I
  */
 JNIEXPORT jint JNICALL
@@ -601,8 +609,8 @@ Java_org_linkgrammar_LinkGrammar_getLinkageLinkCost(JNIEnv *env, jclass cls)
 }
 
 /*
- * Class:		 LinkGrammar
- * Method:		getNumLinks
+ * Class:      LinkGrammar
+ * Method:     getNumLinks
  * Signature: ()I
  */
 JNIEXPORT jint JNICALL
@@ -613,8 +621,8 @@ Java_org_linkgrammar_LinkGrammar_getNumLinks(JNIEnv *env, jclass cls)
 }
 
 /*
- * Class:		 LinkGrammar
- * Method:		getLinkLWord
+ * Class:      LinkGrammar
+ * Method:     getLinkLWord
  * Signature: (I)I
  */
 JNIEXPORT jint JNICALL
@@ -625,8 +633,8 @@ Java_org_linkgrammar_LinkGrammar_getLinkLWord(JNIEnv *env, jclass cls, jint i)
 }
 
 /*
- * Class:		 LinkGrammar
- * Method:		getLinkRWord
+ * Class:      LinkGrammar
+ * Method:     getLinkRWord
  * Signature: (I)I
  */
 JNIEXPORT jint JNICALL
@@ -637,8 +645,8 @@ Java_org_linkgrammar_LinkGrammar_getLinkRWord(JNIEnv *env, jclass cls, jint i)
 }
 
 /*
- * Class:		 LinkGrammar
- * Method:		getLinkLLabel
+ * Class:      LinkGrammar
+ * Method:     getLinkLLabel
  * Signature: (I)Ljava/lang/String;
  */
 JNIEXPORT jstring JNICALL
@@ -652,8 +660,8 @@ Java_org_linkgrammar_LinkGrammar_getLinkLLabel(JNIEnv *env, jclass cls, jint i)
 }
 
 /*
- * Class:		 LinkGrammar
- * Method:		getLinkRLabel
+ * Class:      LinkGrammar
+ * Method:     getLinkRLabel
  * Signature: (I)Ljava/lang/String;
  */
 JNIEXPORT jstring JNICALL
@@ -667,8 +675,8 @@ Java_org_linkgrammar_LinkGrammar_getLinkRLabel(JNIEnv *env, jclass cls, jint i)
 }
 
 /*
- * Class:		 LinkGrammar
- * Method:		getLinkLabel
+ * Class:      LinkGrammar
+ * Method:     getLinkLabel
  * Signature: (I)Ljava/lang/String;
  */
 JNIEXPORT jstring JNICALL
@@ -682,8 +690,8 @@ Java_org_linkgrammar_LinkGrammar_getLinkLabel(JNIEnv *env, jclass cls, jint i)
 }
 
 /*
- * Class:		 LinkGrammar
- * Method:		constituentString
+ * Class:      LinkGrammar
+ * Method:     constituentString
  * Signature: ()Ljava/lang/String;
  */
 JNIEXPORT jstring JNICALL
@@ -702,8 +710,8 @@ Java_org_linkgrammar_LinkGrammar_getConstituentString(JNIEnv *env, jclass cls)
 }
 
 /*
- * Class:		 LinkGrammar
- * Method:		linkString
+ * Class:      LinkGrammar
+ * Method:     linkString
  * Signature: ()Ljava/lang/String;
  */
 JNIEXPORT jstring JNICALL
@@ -717,8 +725,8 @@ Java_org_linkgrammar_LinkGrammar_getLinkString(JNIEnv *env, jclass cls)
 }
 
 /*
- * Class:		 LinkParser
- * Method:		isPastTenseForm
+ * Class:      LinkParser
+ * Method:     isPastTenseForm
  * Signature: (Ljava/lang/String;)Z
  *
  * @deprecated -- past-tense verbs are tagged with .v-d or .w-d or .q-d
@@ -738,8 +746,8 @@ Java_org_linkgrammar_LinkGrammar_isPastTenseForm(JNIEnv *env, jclass cls, jstrin
 }
 
 /*
- * Class:		 LinkParser
- * Method:		isEntity
+ * Class:      LinkParser
+ * Method:     isEntity
  * Signature: (Ljava/lang/String;)Z
  */
 JNIEXPORT jboolean JNICALL
