@@ -130,6 +130,10 @@ link_public_api(void)
      parse_options_set_cost_model_type(Parse_Options opts, int cm);
 link_public_api(int)
      parse_options_get_cost_model_type(Parse_Options opts);
+link_public_api(void)
+     parse_options_set_use_fat_links(Parse_Options opts, int use_fat_links);
+link_public_api(int)
+     parse_options_get_use_fat_links(Parse_Options opts);
 link_public_api(void)
      parse_options_set_use_sat_parser(Parse_Options opts, int use_sat_solver);
 link_public_api(int)
@@ -247,6 +251,8 @@ link_public_api(int)
      sentence_num_valid_linkages(Sentence sent);
 link_public_api(int)
      sentence_num_linkages_post_processed(Sentence sent);
+link_public_api(int)
+     sentence_num_thin_linkages(Sentence sent);
 link_public_api(int)
      sentence_num_violations(Sentence sent, int i);
 link_public_api(int)
@@ -257,7 +263,7 @@ link_public_api(int)
      sentence_link_cost(Sentence sent, int i);
 
 link_public_api(int)
-     sentence_nth_word_has_disjunction(Sentence sent, int i);
+     sentence_contains_conjunction(Sentence sent);
 
 /**********************************************************************
 *
@@ -339,6 +345,8 @@ link_public_api(void)
      linkage_free_diagram(char * str);
 link_public_api(int)
      linkage_compute_union(Linkage linkage);
+link_public_api(int)
+     linkage_is_fat(Linkage linkage);
 link_public_api(int)
      linkage_unused_word_cost(Linkage linkage);
 link_public_api(int)
@@ -429,6 +437,10 @@ link_public_api(void)
 link_public_api(const char *)
      sentence_get_nth_word(Sentence sent, int i);
 
+/* Who uses this function, anyway? How did this get exported? */
+link_public_api(int)
+     sentence_nth_word_has_disjunction(Sentence sent, int i);
+
 /* This is not intended for general use; its specific to the internals
  * of the command-line client.  It was exported by accident. */
 link_public_api(void)
