@@ -77,7 +77,7 @@ static int verbosity = 0;
 typedef enum
 {
 	UNGRAMMATICAL='*',
-	PARSE_WITH_DISJUNCT_COST_GT_0=':',
+	PARSE_WITH_DISJUNCT_COST_GT_0=':',  /* Not used anywhere, currently ... */
 	NO_LABEL=' '
 } Label;
 
@@ -339,18 +339,20 @@ static int process_some_linkages(Sentence sent, Parse_Options opts)
 			corpus_cost = linkage_corpus_cost(linkage);
 			if (corpus_cost < 0.0f)
 			{
-				fprintf(stdout, "cost vector = (UNUSED=%d DIS=%d AND=%d LEN=%d)\n",
+				fprintf(stdout, "cost vector = (UNUSED=%d DIS=%d FAT=%d AND=%d LEN=%d)\n",
 				       linkage_unused_word_cost(linkage),
 				       linkage_disjunct_cost(linkage),
+				       linkage_is_fat(linkage),
 				       linkage_and_cost(linkage),
 				       linkage_link_cost(linkage));
 			}
 			else
 			{
-				fprintf(stdout, "cost vector = (CORP=%6.4f UNUSED=%d DIS=%d AND=%d LEN=%d)\n",
+				fprintf(stdout, "cost vector = (CORP=%6.4f UNUSED=%d DIS=%d FAT=%d AND=%d LEN=%d)\n",
 				       corpus_cost,
 				       linkage_unused_word_cost(linkage),
 				       linkage_disjunct_cost(linkage),
+				       linkage_is_fat(linkage),
 				       linkage_and_cost(linkage),
 				       linkage_link_cost(linkage));
 			}
@@ -404,6 +406,9 @@ static void batch_process_some_linkages(Label label,
 	Linkage linkage;
 
 	if (there_was_an_error(label, sent, opts)) {
+		/* Note: sentence_num_linkages_found returns total linkages
+		 * not valid linkages. So the printed linkage might be bad... 
+		 */
 		if (sentence_num_linkages_found(sent) > 0) {
 			linkage = linkage_create(0, sent, opts);
 			process_linkage(linkage, opts);
@@ -453,6 +458,7 @@ static void setup_panic_parse_options(Parse_Options opts)
 	parse_options_set_min_null_count(opts, 1);
 	parse_options_set_max_null_count(opts, MAX_SENTENCE);
 	parse_options_set_max_parse_time(opts, 60);
+	parse_options_set_use_fat_links(opts, 0);
 	parse_options_set_islands_ok(opts, 1);
 	parse_options_set_short_length(opts, 6);
 	parse_options_set_all_short_connectors(opts, 1);
@@ -698,6 +704,26 @@ int main(int argc, char * argv[])
 		num_linkages = sentence_parse(sent, opts);
 		if (num_linkages < 0) continue;
 
+#if 0
+		/* Try again, this time ommitting the requirement for
+		 * definite articles, etc. This should allow for the parsing
+		 * of newspaper headlines and other clipped speech.
+		 *
+		 * XXX Unfortunately, this also allows for the parsing of
+		 * all sorts of ungrammatical sentences which should not 
+		 * parse, and leads to bad parses of many other unparsable
+		 * but otherwise grammatical sentences.  Thus, this trick
+		 * pretty much fails; we leave it here to document the 
+		 * experiment.
+		 */
+		if (num_linkages == 0)
+		{
+			parse_options_set_disjunct_costf(opts, 3.5f);
+			num_linkages = sentence_parse(sent, opts);
+			if (num_linkages < 0) continue;
+		}
+#endif
+
 		/* Try using a larger list of disjuncts */
 		if ((num_linkages == 0) && parse_options_get_use_cluster_disjuncts(opts))
 		{
@@ -748,6 +774,7 @@ int main(int argc, char * argv[])
 			parse_options_get_panic_mode(opts))
 		{
 			/* print_total_time(opts); */
+			batch_errors++;
 			if (verbosity > 0) fprintf(stdout, "Entering \"panic\" mode...\n");
 			parse_options_reset_resources(panic_parse_opts);
 			parse_options_set_verbosity(panic_parse_opts, verbosity);
