@@ -80,10 +80,12 @@ void resources_reset(Resources r)
 	r->memory_exhausted = FALSE;
 }
 
+#if 0
 static void resources_reset_time(Resources r)
 {
 	r->when_last_called = r->time_when_parse_started = current_usage_time();
 }
+#endif
 
 void resources_reset_space(Resources r)
 {
@@ -121,7 +123,8 @@ static void resources_print_time(int verbosity, Resources r, const char * s)
 	new_t = current_usage_time();
 	if (verbosity > 1) {
 		printf("++++");
-		left_print_string(stdout, s, "										   ");
+		left_print_string(stdout, s,
+			"                                     ");
 		printf("%7.2f seconds\n", new_t - r->when_last_called);
 	}
 	r->when_last_called = new_t;
